@@ -11,61 +11,18 @@ from gi.repository import Gtk, GLib, Pango, Gdk
 from tempfile import NamedTemporaryFile
 from quodlibet.plugins.playlist import PLAYLIST_HANDLER
 
-from util import *
-
 from quodlibet import config
 from quodlibet import const
 from quodlibet import qltk
-from quodlibet import util
 from quodlibet.browsers._base import Browser
 from quodlibet.formats._audio import AudioFile
 from quodlibet.util.collection import Playlist
+from quodlibet.util import connect_obj
 from quodlibet.qltk.songsmenu import SongsMenu
 from quodlibet.qltk.views import RCMHintedTreeView
-from quodlibet.qltk.x import ScrolledWindow, Alignment, SeparatorMenuItem
-from quodlibet.util.dprint import print_d
+from quodlibet.qltk.x import ScrolledWindow, Alignment
 
-
-class Menu(Gtk.Menu):
-    def __init__(self, songs, parent=None):
-        super(Menu, self).__init__()
-        i = Gtk.MenuItem(label=_("_New Playlist"), use_underline=True)
-        i.connect_object(
-            'activate', self.__add_to_playlist, None, songs, parent)
-        self.append(i)
-        self.append(SeparatorMenuItem())
-        self.set_size_request(int(i.size_request().width * 2), -1)
-
-        for playlist in PlaylistsBrowser.playlists():
-            name = playlist.name
-            i = Gtk.CheckMenuItem(name)
-            some, all = playlist.has_songs(songs)
-            i.set_active(some)
-            i.set_inconsistent(some and not all)
-            i.get_child().set_ellipsize(Pango.EllipsizeMode.END)
-            i.connect_object(
-                'activate', self.__add_to_playlist, playlist, songs, parent)
-            self.append(i)
-
-    def __add_to_playlist(playlist, songs, parent):
-        if playlist is None:
-            if len(songs) == 1:
-                title = songs[0].comma("title")
-            else:
-                title = ngettext(
-                    "%(title)s and %(count)d more",
-                    "%(title)s and %(count)d more",
-                    len(songs) - 1) % {
-                        'title': songs[0].comma("title"),
-                        'count': len(songs) - 1
-                    }
-            title = GetPlaylistName(qltk.get_top_parent(parent)).run(title)
-            if title is None:
-                return
-            playlist = Playlist.new(PLAYLISTS, title)
-        playlist.extend(songs)
-        PlaylistsBrowser.changed(playlist)
-    __add_to_playlist = staticmethod(__add_to_playlist)
+from .util import *
 
 
 DND_QL, DND_URI_LIST, DND_MOZ_URL = range(3)
@@ -80,7 +37,8 @@ class PlaylistsBrowser(Gtk.VBox, Browser):
     replaygain_profiles = ["track"]
 
     def pack(self, songpane):
-        container = qltk.RHPaned()
+        container = qltk.ConfigRHPaned(
+            "browsers", "playlistsbrowser_pos", 0.4)
         self.show()
         container.pack1(self, True, False)
         container.pack2(songpane, True, False)
@@ -93,7 +51,7 @@ class PlaylistsBrowser(Gtk.VBox, Browser):
     @classmethod
     def init(klass, library):
         model = klass.__lists.get_model()
-        for playlist in os.listdir(util.fsnative(PLAYLISTS)):
+        for playlist in os.listdir(PLAYLISTS):
             try:
                 playlist = Playlist(PLAYLISTS, Playlist.unquote(playlist),
                                     library=library)
@@ -124,7 +82,7 @@ class PlaylistsBrowser(Gtk.VBox, Browser):
     @classmethod
     def __removed(klass, library, songs):
         for playlist in klass.playlists():
-            if playlist.remove_songs(songs, library):
+            if playlist.remove_songs(songs):
                 PlaylistsBrowser.changed(playlist)
 
     @classmethod
@@ -149,21 +107,25 @@ class PlaylistsBrowser(Gtk.VBox, Browser):
 
     def Menu(self, songs, songlist, library):
         menu = super(PlaylistsBrowser, self).Menu(songs, songlist, library)
-        model, rows = songlist.get_selection().get_selected_rows()
-        iters = map(model.get_iter, rows)
+        model, iters = self.__get_selected_songs(songlist)
         i = qltk.MenuItem(_("_Remove from Playlist"), Gtk.STOCK_REMOVE)
-        i.connect_object('activate', self.__remove, iters, model)
+        qltk.add_fake_accel(i, "Delete")
+        connect_obj(i, 'activate', self.__remove, iters, model)
         i.set_sensitive(bool(self.__view.get_selection().get_selected()[1]))
         menu.preseparate()
         menu.prepend(i)
         return menu
 
+    def __get_selected_songs(self, songlist):
+        model, rows = songlist.get_selection().get_selected_rows()
+        iters = map(model.get_iter, rows)
+        return model, iters
+
     __lists = Gtk.TreeModelSort(model=Gtk.ListStore(object))
     __lists.set_default_sort_func(lambda m, a, b, data: cmp(m[a][0], m[b][0]))
 
-    def __init__(self, library, main):
+    def __init__(self, library):
         super(PlaylistsBrowser, self).__init__(spacing=6)
-        self.__main = main
         self.__view = view = RCMHintedTreeView()
         self.__view.set_enable_search(True)
         self.__view.set_search_column(0)
@@ -214,20 +176,30 @@ class PlaylistsBrowser(Gtk.VBox, Browser):
         view.connect('drag-data-get', self.__drag_data_get)
         view.connect('drag-motion', self.__drag_motion)
         view.connect('drag-leave', self.__drag_leave)
-        if main:
-            view.connect('row-activated', lambda *x: self.emit("activated"))
-        else:
-            render.set_property('editable', True)
+
+        view.connect('row-activated', lambda *x: self.emit("activated"))
+
         view.get_selection().connect('changed', self.activate)
 
         s = view.get_model().connect('row-changed', self.__check_current)
-        self.connect_object('destroy', view.get_model().disconnect, s)
+        connect_obj(self, 'destroy', view.get_model().disconnect, s)
 
         self.connect('key-press-event', self.__key_pressed)
 
         for child in self.get_children():
             child.show_all()
 
+    def key_pressed(self, event):
+        if qltk.is_accel(event, "Delete"):
+            self.__handle_songlist_delete()
+            return True
+        return False
+
+    def __handle_songlist_delete(self, *args):
+        songlist = qltk.get_top_parent(self).songlist
+        model, iters = self.__get_selected_songs(songlist)
+        self.__remove(iters, model)
+
     def __key_pressed(self, widget, event):
         if qltk.is_accel(event, "Delete"):
             model, iter = self.__view.get_selection().get_selected()
@@ -276,6 +248,8 @@ class PlaylistsBrowser(Gtk.VBox, Browser):
             for iter_remove in iters:
                 smodel.remove(iter_remove)
             playlist = model[iter][0]
+            # Calling playlist.remove_songs(songs) won't remove the right ones
+            # if there are duplicates
             playlist.clear()
             playlist.extend([row[0] for row in smodel])
             PlaylistsBrowser.changed(playlist)
@@ -373,7 +347,7 @@ class PlaylistsBrowser(Gtk.VBox, Browser):
                     model.convert_iter_to_child_iter(itr))
 
         rem = Gtk.ImageMenuItem(Gtk.STOCK_DELETE, use_stock=True)
-        rem.connect_object('activate', _remove, model, itr)
+        connect_obj(rem, 'activate', _remove, model, itr)
         menu.prepend(rem)
 
         def _rename(path):
@@ -382,7 +356,7 @@ class PlaylistsBrowser(Gtk.VBox, Browser):
 
         ren = qltk.MenuItem(_("_Rename"), Gtk.STOCK_EDIT)
         qltk.add_fake_accel(ren, "F2")
-        ren.connect_object('activate', _rename, model.get_path(itr))
+        connect_obj(ren, 'activate', _rename, model.get_path(itr))
         menu.prepend(ren)
 
         playlist = model[itr][0]
@@ -418,7 +392,6 @@ class PlaylistsBrowser(Gtk.VBox, Browser):
         else:
             row = self.__lists[path]
             self.__lists.row_changed(row.path, row.iter)
-        render.set_property('editable', not self.__main)
 
     def __import(self, activator, library):
         filt = lambda fn: fn.endswith(".pls") or fn.endswith(".m3u")
