@@ -1,3 +1,11 @@
 from quodlibet.parse._query import Query
 from quodlibet.parse._pattern import (Pattern, FileFromPattern, XMLFromPattern,
     XMLFromMarkupPattern, pattern_from_markup)
+
+
+Query
+Pattern
+FileFromPattern
+XMLFromPattern
+XMLFromMarkupPattern
+pattern_from_markup
