@@ -16,14 +16,14 @@ from quodlibet import util
 from quodlibet.qltk.bookmarks import EditBookmarksPane
 from quodlibet.qltk.cover import CoverImage
 from quodlibet.qltk.lyrics import LyricsPane
-from quodlibet.qltk.x import Window
-from quodlibet.qltk.window import PersistentWindowMixin
-from quodlibet.util import tag
+from quodlibet.qltk.window import Window, PersistentWindowMixin
+from quodlibet.util import tag, connect_destroy
+from quodlibet.util.tags import readable
 from quodlibet.util.path import fsdecode, filesize, unexpand
 
 
-def Label(*args):
-    l = Gtk.Label(*args)
+def Label(label=None):
+    l = Gtk.Label(label=label)
     l.set_selectable(True)
     l.set_alignment(0, 0)
     return l
@@ -48,34 +48,9 @@ def SW():
     return swin
 
 
-class ErrorPane(Gtk.VBox):
-    def __init__(self, song):
-        super(ErrorPane, self).__init__(spacing=6)
-        self.set_border_width(12)
-        sw = Gtk.ScrolledWindow()
-        view = Gtk.TextView()
-        view.get_buffer().set_text(song.get("~errors", ""))
-        view.set_wrap_mode(Gtk.WrapMode.WORD)
-        sw.add(view)
-        sw.set_policy(Gtk.PolicyType.AUTOMATIC, Gtk.PolicyType.AUTOMATIC)
-        sw.set_shadow_type(Gtk.ShadowType.IN)
-        button = qltk.Button(_("_Clear Errors"), Gtk.STOCK_DIALOG_ERROR)
-        button.connect('clicked', self.__clear_errors, song, view)
-        view.set_editable(False)
-        self.pack_start(sw, True, True, 0)
-        self.pack_start(button, False, True, 0)
-
-    def __clear_errors(self, button, song, view):
-        try:
-            del(song["~errors"])
-        except KeyError:
-            pass
-        view.get_buffer().set_text("")
-
-
 class NoSongs(Gtk.Label):
     def __init__(self):
-        super(NoSongs, self).__init__(_("No songs are selected."))
+        super(NoSongs, self).__init__(label=_("No songs are selected."))
         self.title = _("No Songs")
 
 
@@ -102,13 +77,7 @@ class OneSong(qltk.Notebook):
         bookmarks.set_border_width(12)
         self.append_page(bookmarks)
 
-        if "~errors" in song:
-            errors = ErrorPane(song)
-            errors.title = _("Errors")
-            self.append_page(errors)
-
-        s = library.connect('changed', self.__check_changed, vbox, song)
-        self.connect_object('destroy', library.disconnect, s)
+        connect_destroy(library, 'changed', self.__check_changed, vbox, song)
 
     def __check_changed(self, library, songs, vbox, song):
         if song in songs:
@@ -193,20 +162,15 @@ class OneSong(qltk.Notebook):
             vb.pack_start(l, False, True, 0)
         else:
             title = tag("~people")
-        for names, tag_ in [
-            ("performers", "performer"),
-            ("lyricists", "lyricist"),
-            ("arrangers", "arranger"),
-            ("composers", "composer"),
-            ("conductors", "conductor"),
-            ("authors", "author")]:
+        for tag_ in ["performer", "lyricist", "arranger", "composer",
+                     "conductor", "author"]:
             if tag_ in song:
                 l = Label(song[tag_])
                 l.set_ellipsize(Pango.EllipsizeMode.END)
                 if len(song.list(tag_)) == 1:
                     name = tag(tag_)
                 else:
-                    name = _(names)
+                    name = readable(tag_, plural=True)
                 vb.pack_start(Frame(util.capitalize(name), l), False, False, 0)
         performers = {}
         for tag_ in song:
@@ -262,22 +226,25 @@ class OneSong(qltk.Notebook):
             lastplayed = _("Never")
         added = ftime(song.get("~#added", 0))
         rating = song("~rating")
+        has_rating = "~#rating" in song
 
-        t = Gtk.Table(5, 2)
+        t = Gtk.Table(n_rows=5, n_columns=2)
         t.set_col_spacings(6)
         t.set_homogeneous(False)
-        table = [(_("added"), added),
-                 (_("last played"), lastplayed),
-                 (_("plays"), playcount),
-                 (_("skips"), skipcount),
-                 (_("rating"), rating)]
+        table = [(_("added"), added, True),
+                 (_("last played"), lastplayed, True),
+                 (_("plays"), playcount, True),
+                 (_("skips"), skipcount, True),
+                 (_("rating"), rating, has_rating)]
 
-        for i, (l, r) in enumerate(table):
+        for i, (l, r, s) in enumerate(table):
             l = "<b>%s</b>" % util.capitalize(util.escape(l) + ":")
             lab = Label()
             lab.set_markup(l)
             t.attach(lab, 0, 1, i + 1, i + 2, xoptions=Gtk.AttachOptions.FILL)
-            t.attach(Label(r), 1, 2, i + 1, i + 2)
+            label = Label(r)
+            label.set_sensitive(s)
+            t.attach(label, 1, 2, i + 1, i + 2)
 
         box.pack_start(Frame(_("Library"), t), False, False, 0)
 
@@ -300,7 +267,7 @@ class OneSong(qltk.Notebook):
         else:
             bitrate = False
 
-        t = Gtk.Table(4, 2)
+        t = Gtk.Table(n_rows=4, n_columns=2)
         t.set_col_spacings(6)
         t.set_homogeneous(False)
         table = [(_("length"), length),
@@ -518,7 +485,7 @@ class OneArtist(qltk.Notebook):
         box.pack_start(Frame(_("Selected Discography"), l), False, False, 0)
 
         covers = [ac for ac in covers if bool(ac[1])]
-        t = Gtk.Table(4, (len(covers) // 4) + 1)
+        t = Gtk.Table(n_rows=4, n_columns=(len(covers) // 4) + 1)
         t.set_col_spacings(12)
         t.set_row_spacings(12)
         added = set()
@@ -603,7 +570,7 @@ class ManySongs(qltk.Notebook):
                 size += filesize(song["~filename"])
             except EnvironmentError:
                 pass
-        table = Gtk.Table(2, 2)
+        table = Gtk.Table(n_rows=2, n_columns=2)
         table.set_col_spacings(6)
         table.attach(Label(_("Total length:")), 0, 1, 0, 1,
                      xoptions=Gtk.AttachOptions.FILL)
@@ -618,15 +585,12 @@ class ManySongs(qltk.Notebook):
 class Information(Window, PersistentWindowMixin):
     def __init__(self, library, songs, parent=None):
         super(Information, self).__init__(dialog=False)
-        self.set_border_width(12)
         self.set_default_size(400, 400)
         self.enable_window_tracking("quodlibet_information")
         if len(songs) > 1:
-            sig = library.connect('changed', self.__check_changed)
-            self.connect_object('destroy', library.disconnect, sig)
+            connect_destroy(library, 'changed', self.__check_changed)
         if len(songs) > 0:
-            sig = library.connect('removed', self.__check_removed)
-            self.connect_object('destroy', library.disconnect, sig)
+            connect_destroy(library, 'removed', self.__check_removed)
         self.__songs = songs
         self.__update(library)
         self.set_transient_for(qltk.get_top_parent(parent))
