@@ -5,6 +5,7 @@
 # it under the terms of the GNU General Public License version 2 as
 # published by the Free Software Foundation
 
+import os
 import contextlib
 from cStringIO import StringIO
 
@@ -12,27 +13,45 @@ from gi.repository import Gtk, Gdk, GObject, Pango, GLib
 import cairo
 
 from quodlibet import config
-from quodlibet.qltk import get_top_parent, is_accel, is_wayland
+from quodlibet.qltk import get_top_parent, is_accel, is_wayland, gtk_version, \
+    menu_popup
+from quodlibet.qltk.image import get_scale_factor
 
 
 class TreeViewHints(Gtk.Window):
     """Handle 'hints' for treeviews. This includes expansions of truncated
     columns, and in the future, tooltips."""
 
+    class _MinLabel(Gtk.Label):
+
+        def do_get_preferred_width(*args):
+            return (0, Gtk.Label.do_get_preferred_width(*args)[0])
+
     # input_shape_combine_region does not work under Windows, we have
     # to pass all events to the treeview. In case it does work, this handlers
     # will never be called.
     __gsignals__ = dict.fromkeys(
         ['button-press-event', 'button-release-event',
-         'motion-notify-event', 'scroll-event'],
+         'motion-notify-event', 'scroll-event',
+         'enter-notify-event', 'leave-notify-event'],
         'override')
 
     def __init__(self):
         super(TreeViewHints, self).__init__(type=Gtk.WindowType.POPUP)
-        self.__label = label = Gtk.Label()
+        self.__clabel = Gtk.Label()
+        self.__clabel.show()
+        self.__clabel.set_alignment(0, 0.5)
+        self.__clabel.set_ellipsize(Pango.EllipsizeMode.NONE)
+
+        screen = self.get_screen()
+        rgba = screen.get_rgba_visual()
+        if rgba is not None:
+            self.set_visual(rgba)
+
+        self.__label = label = self._MinLabel()
         label.set_alignment(0, 0.5)
-        label.show()
         label.set_ellipsize(Pango.EllipsizeMode.NONE)
+        label.show()
         self.add(label)
 
         self.add_events(
@@ -44,31 +63,56 @@ class TreeViewHints(Gtk.Window):
             Gdk.EventMask.ENTER_NOTIFY_MASK |
             Gdk.EventMask.LEAVE_NOTIFY_MASK |
             Gdk.EventMask.SCROLL_MASK |
-            Gdk.EventMask.POINTER_MOTION_MASK |
-            Gdk.EventMask.POINTER_MOTION_HINT_MASK)
+            Gdk.EventMask.POINTER_MOTION_MASK)
 
         context = self.get_style_context()
         context.add_class("tooltip")
+        context.add_class("ql-tooltip")
 
         self.set_accept_focus(False)
         self.set_resizable(False)
         self.set_name("gtk-tooltip")
-        self.set_border_width(1)
+        if gtk_version < (3, 13):
+            self.set_border_width(1)
+
         self.connect('leave-notify-event', self.__undisplay)
 
         self.__handlers = {}
         self.__current_path = self.__current_col = None
         self.__current_renderer = None
         self.__view = None
+        self.__hide_id = None
 
     def connect_view(self, view):
+
+        # don't depend on padding set by theme, we need the text coordinates
+        # to match in all cases
+        self._style_provider = style_provider = Gtk.CssProvider()
+        style_provider.load_from_data("""
+            .ql-tooltip * {
+                border-width: 0px;
+                padding: 0px;
+            }
+            .ql-tooltip {
+                padding: 0px;
+            }
+        """)
+
+        # somehow this doesn't apply if we set it on the window, only
+        # if set for the screen. gets reverted again in disconnect_view()
+        Gtk.StyleContext.add_provider_for_screen(
+            Gdk.Screen.get_default(),
+            style_provider,
+            Gtk.STYLE_PROVIDER_PRIORITY_APPLICATION
+        )
+
         self.__handlers[view] = [
             view.connect('motion-notify-event', self.__motion),
+            view.connect('leave-notify-event', self.__motion),
             view.connect('scroll-event', self.__undisplay),
             view.connect('key-press-event', self.__undisplay),
             view.connect('unmap', self.__undisplay),
             view.connect('destroy', self.disconnect_view),
-            view.connect('leave-notify-event', self.__leave_undisplay),
         ]
 
     def disconnect_view(self, view):
@@ -83,8 +127,17 @@ class TreeViewHints(Gtk.Window):
             self.__undisplay()
         self.set_transient_for(None)
 
+        if self.__hide_id:
+            GLib.source_remove(self.__hide_id)
+            self.__hide_id = None
+            self.hide()
+
+        Gtk.StyleContext.remove_provider_for_screen(
+            Gdk.Screen.get_default(), self._style_provider)
+
     def __motion(self, view, event):
         label = self.__label
+        clabel = self.__clabel
 
         # trigger over row area, not column headers
         if event.window is not view.get_bin_window():
@@ -105,9 +158,9 @@ class TreeViewHints(Gtk.Window):
             self.__undisplay()
             return False
 
-        area = view.get_cell_area(path, col)
+        col_area = view.get_cell_area(path, col)
         # make sure we are on the same level
-        if x < area.x:
+        if x < col_area.x:
             self.__undisplay()
             return False
 
@@ -127,17 +180,27 @@ class TreeViewHints(Gtk.Window):
 
         if self.__current_renderer == renderer and self.__current_path == path:
             return False
-        else:
-            self.__undisplay()
 
         # only ellipsized text renderers
         if not isinstance(renderer, Gtk.CellRendererText):
+            self.__undisplay()
             return False
-        if renderer.get_property('ellipsize') == Pango.EllipsizeMode.NONE:
+
+        ellipsize = renderer.get_property('ellipsize')
+        if ellipsize == Pango.EllipsizeMode.END:
+            expand_left = False
+        elif ellipsize == Pango.EllipsizeMode.MIDDLE:
+            # depending on where the cursor is
+            expand_left = x > col_area.x + render_offset + render_width / 2
+        elif ellipsize == Pango.EllipsizeMode.START:
+            expand_left = True
+        else:
+            self.__undisplay()
             return False
 
         # don't display if the renderer is in editing mode
         if renderer.props.editing:
+            self.__undisplay()
             return False
 
         # set the cell renderer attributes for the active cell
@@ -148,61 +211,92 @@ class TreeViewHints(Gtk.Window):
         # to be saved on the python side, so we can copy it to the label
         markup = getattr(renderer, "markup", None)
         if markup is None:
-            label.set_text(renderer.get_property('text'))
+            text = renderer.get_property('text')
+            set_text = lambda l: l.set_text(text)
         else:
             # markup can also be column index
             if isinstance(markup, int):
                 markup = model[path][markup]
-            label.set_markup(markup)
+            set_text = lambda l: l.set_markup(markup)
 
         # Use the renderer padding as label padding so the text offset matches
         render_xpad = renderer.get_property("xpad")
-        label.set_ellipsize(Pango.EllipsizeMode.NONE)
-        label.set_padding(render_xpad, 0)
-        # size_request makes sure the layout size got updated
-        label.size_request()
-        label_width = label.get_layout().get_pixel_size()[0]
-        label_width += label.get_layout_offsets()[0] or 0
+
+        # the renderer xpad is not enough for the tooltip, especially with
+        # rounded corners the label gets nearly clipped.
+        MIN_HINT_X_PAD = 4
+        if render_xpad < MIN_HINT_X_PAD:
+            extra_xpad = MIN_HINT_X_PAD - render_xpad
+        else:
+            extra_xpad = 0
+
+        label.set_padding(render_xpad + extra_xpad, 0)
+        set_text(clabel)
+        clabel.set_padding(render_xpad, 0)
+        label_width = clabel.get_layout().get_pixel_size()[0]
+        label_width += clabel.get_layout_offsets()[0] or 0
         # layout offset includes the left padding, so add one more
         label_width += render_xpad
 
         # CellRenderer width is too large if it's the last one in a column.
         # Use cell_area width as a maximum and limit render_width.
-        max_width = view.get_cell_area(path, col).width
+        max_width = col_area.width
         if render_width + render_offset > max_width:
             render_width = max_width - render_offset
 
         # don't display if it doesn't need expansion
         if label_width < render_width:
+            self.__undisplay()
             return False
 
         dummy, ox, oy = view.get_window().get_origin()
+        bg_area = view.get_background_area(path, None)
 
         # save for adjusting passthrough events
-        self.__dx, self.__dy = area.x + render_offset, area.y
+        self.__dx, self.__dy = col_area.x + render_offset, bg_area.y
+        self.__dx -= extra_xpad
+        if expand_left:
+            # shift to the left
+            # FIXME: ellipsize start produces a space at the end depending
+            # on the text. I don't know how to compute it..
+            self.__dx -= (label_width - render_width)
 
         # final window coordinates/size
         x = ox + self.__dx
         y = oy + self.__dy
         x, y = view.convert_bin_window_to_widget_coords(x, y)
 
-        w = label_width
-        h = area.height
+        w = label_width + extra_xpad * 2
+        h = bg_area.height
 
         if not is_wayland():
-            # clip on the right if it's bigger than the screen
+            # clip if it's bigger than the screen
             screen_border = 5  # leave some space
-            space_right = Gdk.Screen.width() - x - w - screen_border
 
-            if space_right < 0:
-                w += space_right
-                label.set_ellipsize(Pango.EllipsizeMode.END)
+            if not expand_left:
+                space_right = Gdk.Screen.width() - x - w - screen_border
+
+                if space_right < 0:
+                    w += space_right
+                    label.set_ellipsize(Pango.EllipsizeMode.END)
+                else:
+                    label.set_ellipsize(Pango.EllipsizeMode.NONE)
             else:
-                label.set_ellipsize(Pango.EllipsizeMode.NONE)
+                space_left = x - screen_border
+                if space_left < 0:
+                    x -= space_left
+                    self.__dx -= space_left
+                    w += space_left
+                    label.set_ellipsize(Pango.EllipsizeMode.START)
+                else:
+                    label.set_ellipsize(Pango.EllipsizeMode.NONE)
+        else:
+            label.set_ellipsize(Pango.EllipsizeMode.NONE)
 
         # Don't show if the resulting tooltip would be smaller
         # than the visible area (if not all is on the display)
         if w < render_width:
+            self.__undisplay()
             return False
 
         self.__view = view
@@ -211,30 +305,22 @@ class TreeViewHints(Gtk.Window):
         self.__current_path = path
         self.__current_col = col
 
+        if self.__hide_id:
+            GLib.source_remove(self.__hide_id)
+            self.__hide_id = None
+
         self.set_transient_for(get_top_parent(view))
+        set_text(label)
         self.set_size_request(w, h)
-        self.resize(w, h)
-        self.move(x, y)
 
-        self.show()
-        return False
+        window = self.get_window()
+        if self.get_visible() and window:
+            window.move_resize(x, y, w, h)
+        else:
+            self.move(x, y)
+            self.resize(w, h)
+            self.show()
 
-    def __leave_undisplay(self, view, event):
-        # We don't get motion-notify events in all cases, so
-        # quickly escaping the view can leave stray tooltips.
-        # leave-notify is triggered when we leave the view but also when we
-        # show the tooltip, so this has to check if the cursor is not on
-        # it to not get in an endless hide/show loop.
-
-        if not self.__view:
-            return False
-
-        device = event.get_device()
-        # see docs: get_window_at_position
-        if device and device.get_device_type() != Gdk.DeviceType.SLAVE:
-            win = device.get_window_at_position()[0]
-            if win and win != self.get_window():
-                self.__undisplay()
         return False
 
     def __undisplay(self, *args):
@@ -246,7 +332,22 @@ class TreeViewHints(Gtk.Window):
         self.__current_renderer = self.__edit_id = None
         self.__current_path = self.__current_col = None
         self.__view = None
-        self.hide()
+
+        def hide():
+            self.__hide_id = None
+            self.hide()
+            return False
+
+        if gtk_version < (3, 13):
+            # https://bugzilla.gnome.org/show_bug.cgi?id=731055
+            # Work around Gnome Shell redraw bugs: it doesn't like
+            # multiple hide()/show(), so we try to reduce calls to hide
+            # by aborting it if the pointer is on a new cell shortly after.
+            self.__hide_id = GLib.timeout_add(20, hide)
+        else:
+            # mutter3.12 and gtk3.14 are a bit broken together, so it's safe
+            # to assume we have a fixed mutter release..
+            hide()
 
     def __event(self, event):
         if not self.__view:
@@ -259,6 +360,28 @@ class TreeViewHints(Gtk.Window):
             if not is_wayland():  # present duplicates windows in weston
                 Gtk.Window.present(get_top_parent(self.__view))
 
+        def translate_enter_leave_event(event):
+            # enter/leave events have different x/y values as motion events
+            # so it makes sense to push them to the underlying view as
+            # additional motion events.
+            # Warning: this may result in motion events outside of the
+            # view window.. ?
+            new_event = Gdk.Event.new(Gdk.EventType.MOTION_NOTIFY)
+            struct = new_event.motion
+            for attr in ["x", "y", "x_root", "y_root", "time", "window",
+                         "state", "send_event"]:
+                setattr(struct, attr, getattr(event.crossing, attr))
+            device = Gtk.get_current_event_device()
+            if device is not None:
+                struct.set_device(device)
+            return new_event
+
+        # FIXME: We should translate motion events on the tooltip
+        # to crossing events for the underlying view.
+        # (I think, no tested) Currently the hover scrollbar stays visible
+        # if the mouse leaves the view through the tooltip without the
+        # knowledge of the view.
+
         type_ = event.type
         real_event = None
         if type_ == Gdk.EventType.BUTTON_PRESS:
@@ -267,6 +390,12 @@ class TreeViewHints(Gtk.Window):
             real_event = event.button
         elif type_ == Gdk.EventType.MOTION_NOTIFY:
             real_event = event.motion
+        elif type_ == Gdk.EventType.ENTER_NOTIFY:
+            event = translate_enter_leave_event(event)
+            real_event = event.motion
+        elif type_ == Gdk.EventType.LEAVE_NOTIFY:
+            event = translate_enter_leave_event(event)
+            real_event = event.motion
 
         if real_event:
             real_event.x += self.__dx
@@ -276,7 +405,7 @@ class TreeViewHints(Gtk.Window):
         # nobody else should tie to any TreeViewHints events ever.
         event.any.window = self.__view.get_bin_window()
 
-        event.put()
+        Gtk.main_do_event(event)
 
         return True
 
@@ -289,6 +418,12 @@ class TreeViewHints(Gtk.Window):
     def do_motion_notify_event(self, event):
         return self.__event(event)
 
+    def do_enter_notify_event(self, event):
+        return self.__event(event)
+
+    def do_leave_notify_event(self, event):
+        return self.__event(event)
+
     def do_scroll_event(self, event):
         return self.__event(event)
 
@@ -566,14 +701,16 @@ class BaseView(Gtk.TreeView):
             column.set_sort_indicator(value)
 
 
-def _get_surface_size(surface):
+def _get_surface_size(surface, scale_factor):
     """Returns (width, height) of a surface or None."""
 
     # X11
     try:
-        return surface.get_width(), surface.get_height()
+        w, h = surface.get_width(), surface.get_height()
     except AttributeError:
         pass
+    else:
+        return w / scale_factor, h / scale_factor
 
     # Everything else: pycairo doesn't expose get_image() so we have
     # do it the ugly way through png
@@ -586,9 +723,11 @@ def _get_surface_size(surface):
         return
     else:
         try:
-            return image_surface.get_width(), image_surface.get_height()
+            w, h = image_surface.get_width(), image_surface.get_height()
         except AttributeError:
             pass
+        else:
+            return w / scale_factor, h / scale_factor
 
 
 class DragIconTreeView(BaseView):
@@ -627,7 +766,8 @@ class DragIconTreeView(BaseView):
         if not icons:
             return
 
-        sizes = [_get_surface_size(s) for s in icons]
+        scale_factor = get_scale_factor(self)
+        sizes = [_get_surface_size(s, scale_factor) for s in icons]
         if None in sizes:
             return
         width = max([s[0] for s in sizes])
@@ -645,7 +785,8 @@ class DragIconTreeView(BaseView):
             height += lh
             height += 6  # padding
 
-        surface = cairo.ImageSurface(cairo.FORMAT_ARGB32, width, height)
+        surface = icons[0].create_similar(
+            cairo.CONTENT_COLOR_ALPHA, width, height)
         ctx = cairo.Context(surface)
 
         # render background
@@ -783,35 +924,55 @@ class RCMTreeView(BaseView):
         else:
             pos_func = None
 
-        menu.popup(None, None, pos_func, None, button, time)
+        # force attach the menu to the view
+        attached_widget = menu.get_attach_widget()
+        if attached_widget != self:
+            if attached_widget is not None:
+                menu.detach()
+            menu.attach_to_widget(self, None)
+
+        menu_popup(menu, None, None, pos_func, None, button, time)
         return True
 
     def __popup_position(self, menu, *args):
         path, col = self.get_cursor()
-        if col is None:
-            col = self.get_column(0)
 
         # get a rectangle describing the cell render area (assume 3 px pad)
         rect = self.get_cell_area(path, col)
-        rect.x += 3
-        rect.width -= 6
-        rect.y += 3
-        rect.height -= 6
-        dummy, dx, dy = self.get_window().get_origin()
-        dy += self.get_bin_window().get_position()[1]
+        padding = 3
+        rect.x += padding
+        rect.width = max(rect.width - padding * 2, 0)
+        rect.y += padding
+        rect.height = max(rect.height - padding * 2, 0)
+
+        x, y = self.get_window().get_origin()[1:]
+        x, y = self.convert_bin_window_to_widget_coords(x + rect.x, y + rect.y)
 
-        # fit menu to screen, aligned per text direction
-        screen_width = Gdk.Screen.width()
-        screen_height = Gdk.Screen.height()
         menu.realize()
         ma = menu.get_allocation()
-        menu_y = rect.y + rect.height + dy
-        if menu_y + ma.height > screen_height and rect.y + dy - ma.height > 0:
-            menu_y = rect.y + dy - ma.height
-        if Gtk.Widget.get_default_direction() == Gtk.TextDirection.LTR:
-            menu_x = min(rect.x + dx, screen_width - ma.width)
+        menu_y = rect.height + y
+        if self.get_direction() == Gtk.TextDirection.LTR:
+            menu_x = x
         else:
-            menu_x = max(0, rect.x + dx - ma.width + rect.width)
+            menu_x = x - ma.width + rect.width
+
+        # on X11/win32 we can use the screen size
+        if not is_wayland():
+
+            # fit menu to screen, aligned per text direction
+            screen = self.get_screen()
+            screen_width = screen.get_width()
+            screen_height = screen.get_height()
+
+            # show above row if no space below and enough above
+            if menu_y + ma.height > screen_height and y - ma.height > 0:
+                menu_y = y - ma.height
+
+            # make sure it's not outside of the screen
+            if self.get_direction() == Gtk.TextDirection.LTR:
+                menu_x = max(0, min(menu_x, screen_width - ma.width))
+            else:
+                menu_x = min(max(0, menu_x), screen_width)
 
         return (menu_x, menu_y, True)  # x, y, move_within_screen
 
@@ -822,13 +983,23 @@ class HintedTreeView(BaseView):
 
     def __init__(self, *args, **kwargs):
         super(HintedTreeView, self).__init__(*args, **kwargs)
-        if not config.state('disable_hints'):
+        if self.supports_hints():
             try:
                 tvh = HintedTreeView.hints
             except AttributeError:
                 tvh = HintedTreeView.hints = TreeViewHints()
             tvh.connect_view(self)
 
+    def supports_hints(self):
+        """If the treeview hints support is enabled. Can be used to
+        display scroll bars instead for example.
+        """
+
+        if "QUODLIBET_NO_HINTS" in os.environ:
+            return False
+
+        return not config.state('disable_hints')
+
 
 class TreeViewColumn(Gtk.TreeViewColumn):
     def __init__(self, title="", *args, **kwargs):
@@ -838,6 +1009,23 @@ class TreeViewColumn(Gtk.TreeViewColumn):
         label.show()
         self.set_widget(label)
 
+        # the button gets created once the widget gets realized
+        self._button = None
+        label.__realize = label.connect('realize', self.__realized)
+        self._tooltip_text = None
+
+    def __realized(self, widget):
+        widget.disconnect(widget.__realize)
+        self._button = widget.get_ancestor(Gtk.Button)
+        self.set_tooltip_text(self._tooltip_text)
+
+    def set_tooltip_text(self, text):
+        if self._button:
+            # gtk3.4: set_tooltip_text didn't allow None
+            self._button.props.tooltip_text = text
+        else:
+            self._tooltip_text = text
+
     def set_use_markup(self, value):
         widget = self.get_widget()
         if isinstance(widget, Gtk.Label):
