@@ -9,10 +9,13 @@
 import os
 import random
 import re
+import ctypes
+import ctypes.util
 import sys
 import traceback
 import urlparse
 import unicodedata
+import threading
 import subprocess
 import webbrowser
 import contextlib
@@ -24,12 +27,10 @@ try:
 except ImportError:
     fcntl = None
 
-from quodlibet.util.path import (fsdecode, fsencode, iscommand, fsnative,
-    expanduser, pathname2url, strip_win32_incompat, is_fsnative)
-from quodlibet.util.string.splitters import split_value
+from quodlibet.util.path import iscommand, is_fsnative
 from quodlibet.util.titlecase import title
 
-from quodlibet.const import FSCODING as fscoding, SUPPORT_EMAIL, COPYRIGHT
+from quodlibet.const import SUPPORT_EMAIL, COPYRIGHT
 from quodlibet.util.dprint import print_d, print_
 
 
@@ -208,6 +209,81 @@ def parse_time(timestr, err=(ValueError, re.error)):
         return 0
 
 
+def validate_query_date(datestr):
+    """Validates a user provided date that can be compared using date_key().
+
+    Returns True id the date is valid.
+    """
+
+    parts = datestr.split("-")
+    if len(parts) > 3:
+        return False
+
+    if len(parts) > 2:
+        try:
+            v = int(parts[2])
+        except ValueError:
+            return False
+        else:
+            if not 1 <= v <= 31:
+                return False
+
+    if len(parts) > 1:
+        try:
+            v = int(parts[1])
+        except ValueError:
+            return False
+        else:
+            if not 1 <= v <= 12:
+                return False
+
+    try:
+        int(parts[0])
+    except ValueError:
+        return False
+
+    return True
+
+
+def date_key(datestr):
+    """Parse a date format y-m-d and returns an undefined integer that
+    can only be used to compare dates.
+
+    In case the date string is invalid the returned value is undefined.
+    """
+
+    # this basically does "2001-02-03" -> 20010203
+
+    default = [0, 1, 1]
+    parts = datestr.split("-")
+    parts += default[len(parts):]
+
+    value = 0
+    for d, p, m in zip(default, parts, (10000, 100, 1)):
+        try:
+            value += int(p) * m
+        except ValueError:
+            # so that "2003-01-" is equal to "2003-01" ..
+            value += d * m
+    return value
+
+
+def parse_date(datestr):
+    """Parses yyyy-mm-dd date format and returns unix time.
+
+    Raises ValueError in case the input couldn't be parsed.
+    """
+
+    import time
+
+    try:
+        frmt = ["%Y", "%Y-%m", "%Y-%m-%d"][datestr.count("-")]
+    except IndexError:
+        raise ValueError
+
+    return time.mktime(time.strptime(datestr, frmt))
+
+
 def format_rating(value, blank=True):
     """Turn a number into a sequence of rating symbols."""
 
@@ -242,6 +318,7 @@ def format_size(size):
 
 def format_time(time):
     """Turn a time value in seconds into hh:mm:ss or mm:ss."""
+
     if time < 0:
         time = abs(time)
         prefix = "-"
@@ -256,6 +333,12 @@ def format_time(time):
         return "%s%d:%02d" % (prefix, time // 60, time % 60)
 
 
+def format_time_display(time):
+    """Like format_time, but will use RATIO instead of a colon to separate"""
+
+    return format_time(time).replace(":", u"\u2236")
+
+
 def format_time_long(time, limit=2):
     """Turn a time value in seconds into x hours, x minutes, etc.
 
@@ -267,15 +350,15 @@ def format_time_long(time, limit=2):
         return _("No time information")
 
     cutoffs = [
-        (60, "%d seconds", "%d second"),
-        (60, "%d minutes", "%d minute"),
-        (24, "%d hours", "%d hour"),
-        (365, "%d days", "%d day"),
-        (None, "%d years", "%d year"),
+        (60, lambda n: ngettext("%d second", "%d seconds", n)),
+        (60, lambda n: ngettext("%d minute", "%d minutes", n)),
+        (24, lambda n: ngettext("%d hour", "%d hours", n)),
+        (365, lambda n: ngettext("%d day", "%d days", n)),
+        (None, lambda n: ngettext("%d year", "%d years", n)),
     ]
 
     time_str = []
-    for divisor, plural, single in cutoffs:
+    for divisor, gettext_partial in cutoffs:
         if time < 1:
             break
         if divisor is None:
@@ -283,7 +366,7 @@ def format_time_long(time, limit=2):
         else:
             time, unit = divmod(time, divisor)
         if unit:
-            time_str.append(ngettext(single, plural, unit) % unit)
+            time_str.append(gettext_partial(unit) % unit)
     time_str.reverse()
 
     if limit:
@@ -292,15 +375,6 @@ def format_time_long(time, limit=2):
     return ", ".join(time_str)
 
 
-def split_scan_dirs(s):
-    """Split the value of the "scan" setting, accounting for drive letters on
-    win32."""
-    if sys.platform == "win32":
-        return filter(None, re.findall(r"[a-zA-Z]:[\\/][^:]*", s))
-    else:
-        return filter(None, s.split(":"))
-
-
 def capitalize(str):
     """Capitalize a string, not affecting any character after the first."""
     return str[:1].upper() + str[1:]
@@ -332,7 +406,7 @@ def human_sort_key(s, normalize=unicodedata.normalize):
 def website(site):
     """Open the given URL in the user's default browser"""
 
-    if os.name == "nt":
+    if os.name == "nt" or sys.platform == "darwin":
         return webbrowser.open(site)
 
     # all commands here return immediately
@@ -489,28 +563,54 @@ class DeferredSignal(object):
     priority and prevents multiple calls from being inserted in the
     mainloop at a time, greatly improving responsiveness in some places.
 
+    When the target function will finally be called the arguments passed
+    are the last arguments passed to DeferredSignal.
+
+    If `owner` is given, it will not call the target after the owner is
+    destroyed.
+
     Example usage:
 
     def func(widget, user_arg):
         pass
-    widget.connect('signal', DeferredSignal(func), user_arg)
+    widget.connect('signal', DeferredSignal(func, owner=widget), user_arg)
     """
 
-    def __init__(self, func, timeout=None):
+    def __init__(self, func, timeout=None, owner=None):
         """timeout in milliseconds"""
 
         self.func = func
         self.dirty = False
         self.args = None
 
+        if owner:
+            def destroy_cb(owner):
+                self.abort()
+            owner.connect("destroy", destroy_cb)
+
         from gi.repository import GLib
         if timeout is None:
             self.do_idle_add = GLib.idle_add
         else:
             self.do_idle_add = lambda f: GLib.timeout_add(timeout, f)
 
-    def destroy(self):
-        """Abort any queued up up signal calls"""
+    @property
+    def im_self(self):
+        return self.func.im_self
+
+    @property
+    def __code__(self):
+        return self.func.__code__
+
+    @property
+    def __closure__(self):
+        return self.func.__closure__
+
+    def abort(self):
+        """Abort any queued up calls.
+
+        Can still be reused afterwards.
+        """
 
         if self.dirty:
             from gi.repository import GLib
@@ -528,33 +628,77 @@ class DeferredSignal(object):
         self.func(*self.args)
         self.dirty = False
         self.args = None
+        return False
 
 
-def gobject_weak(fun, *args, **kwargs):
-    """Connect to a signal and disconnect if destroy gets emitted.
-    If parent is given, it connects to its destroy signal
-    Example:
-        gobject_weak(gobject_1.connect, 'changed', self.__changed)
-        gobject_weak(gobject_1.connect, 'changed', self.__changed,
-            parent=gobject_2)
+def connect_obj(this, detailed_signal, handler, that, *args, **kwargs):
+    """A wrapper for connect() that has the same interface as connect_object().
+    Used as a temp solution to get rid of connect_object() calls which may
+    be changed to match the C version more closely in the future.
+
+    https://git.gnome.org/browse/pygobject/commit/?id=86fb12b3e9b75
+
+    While it's not clear if switching to weak references will break anything,
+    we mainly used this for adjusting the callback signature. So using
+    connect() behind the scenes will keep things working as they are now.
     """
-    parent = kwargs.pop("parent", None)
-    obj = fun.__self__
-    sig = fun(*args)
-    disconnect = lambda obj, handle: obj.disconnect(handle)
-    if parent:
-        parent.connect_object('destroy', disconnect, obj, sig)
+
+    def wrap(this, *args):
+        return handler(that, *args)
+
+    return this.connect(detailed_signal, wrap, *args, **kwargs)
+
+
+def _connect_destroy(sender, func, detailed_signal, handler, *args, **kwargs):
+    """Connect a bound method to a foreign object signal and disconnect
+    if the object the method is bound to emits destroy (Gtk.Widget subclass).
+
+    Also works if the handler is a nested function in a method and
+    references the method's bound object.
+
+    This solves the problem that the sender holds a strong reference
+    to the bound method and the bound to object doesn't get GCed.
+    """
+
+    if hasattr(handler, "im_self"):
+        obj = handler.im_self
     else:
-        obj.connect('destroy', disconnect, sig)
-    return sig
+        # XXX: get the "self" var of the enclosing scope.
+        # Used for nested functions which ref the object but aren't methods.
+        # In case they don't ref "self" normal connect() should be used anyway.
+        index = handler.__code__.co_freevars.index("self")
+        obj = handler.__closure__[index].cell_contents
+
+    assert obj is not sender
+
+    handler_id = func(detailed_signal, handler, *args, **kwargs)
+
+    def disconnect_cb(*args):
+        sender.disconnect(handler_id)
+
+    obj.connect('destroy', disconnect_cb)
+    return handler_id
+
+
+def connect_destroy(sender, *args, **kwargs):
+    return _connect_destroy(sender, sender.connect, *args, **kwargs)
+
+
+def connect_after_destroy(sender, *args, **kwargs):
+    return _connect_destroy(sender, sender.connect_after, *args, **kwargs)
 
 
 class cached_property(object):
     """A read-only @property that is only evaluated once."""
+
     def __init__(self, fget, doc=None):
         self.fget = fget
         self.__doc__ = doc or fget.__doc__
-        self.__name__ = fget.__name__
+        self.__name__ = name = fget.__name__
+        # these get name mangled, so caching wont work unless
+        # we mangle too
+        assert not (name.startswith("__") and not name.endswith("__")), \
+            "can't cache a dunder method"
 
     def __get__(self, obj, cls):
         if obj is None:
@@ -759,3 +903,136 @@ def atomic_save(filename, suffix, mode):
         if fcntl is not None:
             fcntl.flock(fileobj.fileno(), fcntl.LOCK_UN)
         fileobj.close()
+
+
+def load_library(names, shared=True):
+    """Load a ctypes library with a range of names to try.
+
+    Handles direct .so names and library names ["libgpod.so", "gpod"].
+
+    If shared is True can return a shared instance.
+    Raises OSError if not found.
+
+    Returns (library, name)
+    """
+
+    if not names:
+        raise ValueError
+
+    if shared:
+        load_func = lambda n: getattr(ctypes.cdll, n)
+    else:
+        load_func = ctypes.cdll.LoadLibrary
+
+    errors = []
+    for name in names:
+        dlopen_name = name
+        if ".so" not in name and ".dll" not in name:
+            dlopen_name = ctypes.util.find_library(name) or name
+
+        try:
+            return load_func(dlopen_name), name
+        except OSError as e:
+            errors.append(str(e))
+
+    raise OSError("\n".join(errors))
+
+
+def is_main_thread():
+    """If the calling thread is the main one"""
+
+    return threading.current_thread().name == "MainThread"
+
+
+class MainRunnerError(Exception):
+    pass
+
+
+class MainRunner(object):
+    """Schedule a function call in the main loop from a
+    worker thread and wait for the result.
+
+    Make sure to call abort() before the main loop gets destroyed, otherwise
+    the worker thread may block forever in call().
+    """
+
+    def __init__(self):
+        self._id = None
+        self._lock = threading.Lock()
+        self._cond = threading.Condition(self._lock)
+        self._return = None
+        self._error = None
+        self._aborted = False
+
+    def _run(self, func, *args, **kwargs):
+        self._error = None
+        try:
+            self._return = func(*args, **kwargs)
+        except Exception as e:
+            self._error = MainRunnerError(e)
+
+    def _idle_run(self, func, *args, **kwargs):
+        with self._lock:
+            try:
+                self._run(func, *args, **kwargs)
+            finally:
+                self._id = None
+                self._cond.notify()
+                return False
+
+    def abort(self):
+        """After this call returns no function will be executed anymore
+        and a currently blocking call will fail with.
+
+        Can be called multiple times and can not fail.
+        call() will always fail after this was called.
+        """
+
+        from gi.repository import GLib
+
+        with self._lock:
+            if self._aborted:
+                return
+            if self._id is not None:
+                GLib.source_remove(self._id)
+                self._id = None
+            self._aborted = True
+            self._error = MainRunnerError("aborted")
+            self._cond.notify()
+
+    def call(self, func, *args, **kwargs):
+        """Runs the function in the main loop and blocks until
+        it is finshed or abort() was called. In case this is called
+        from the main loop the function gets executed immediately.
+
+        The priority kwargs defines the event source priority and will
+        not be passed to func.
+
+        Can raise MainRunnerError in case the function raises an exception
+        or abort() was called before or during this call.
+        """
+
+        from gi.repository import GLib
+
+        with self._lock:
+            if self._aborted:
+                raise self._error
+            # XXX: ideally this should be GLib.MainContext.default().is_owner()
+            # but that's not available in older pygobject
+            if is_main_thread():
+                kwargs.pop("priority", None)
+                self._run(func, *args, **kwargs)
+            else:
+                assert self._id is None
+                self._id = GLib.idle_add(self._idle_run, func, *args, **kwargs)
+                self._cond.wait()
+            if self._error is not None:
+                raise self._error
+            return self._return
+
+
+def re_escape(string, BAD="/.^$*+-?{,\\[]|()<>#=!:"):
+    """A re.escape which also works with unicode"""
+
+    needs_escape = lambda c: (c in BAD and "\\" + c) or c
+    return type(string)().join(map(needs_escape, string))
