@@ -1,6 +1,7 @@
 # -*- coding: utf-8 -*-
 # Copyright 2004-2013 Joe Wreschnig, Michael Urman, Iñigo Serna,
-#                     Christoph Reiter, Steven Robertson, Nick Boultbee
+#                     Christoph Reiter, Steven Robertson
+#           2011-2014 Nick Boultbee
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -8,20 +9,26 @@
 
 from __future__ import absolute_import
 
-from gi.repository import GLib
-
 import os
 import random
 
 from quodlibet import util
 from quodlibet import config
-from quodlibet.formats._audio import PEOPLE, TAG_TO_SORT, INTERN_NUM_DEFAULT
+from quodlibet.formats._audio import TAG_TO_SORT, INTERN_NUM_DEFAULT
+from quodlibet.formats._audio import PEOPLE as _PEOPLE
 from quodlibet.util import thumbnails
 from collections import Iterable
-from quodlibet.util.path import fsencode, escape_filename, unescape_filename
+from quodlibet.util.path import escape_filename, unescape_filename
+from quodlibet.util.path import bytes2fsnative, is_fsnative, fsnative2bytes
+from quodlibet.util.cover.manager import cover_plugins
 from .collections import HashedList
 
 
+PEOPLE = list(_PEOPLE)
+# Collections value albumartist more than song artist (Issue 1034)
+PEOPLE.remove("albumartist")
+PEOPLE.insert(0, "albumartist")
+
 ELPOEP = list(reversed(PEOPLE))
 PEOPLE_SCORE = [100 ** i for i in xrange(len(PEOPLE))]
 
@@ -165,7 +172,8 @@ class Collection(object):
                 w = lambda s: s("~#bitrate", 0) * s("~#length", 0)
                 return sum(w(song) for song in self.songs) / length
             else:
-                # Unknown key. AudioFile will try to cast the values to int,
+                # Standard or unknown numeric key.
+                # AudioFile will try to cast the values to int,
                 # default to avg
                 func = NUM_DEFAULT_FUNCS.get(key, "avg")
 
@@ -182,6 +190,7 @@ class Collection(object):
             return None
         elif key[:1] == "~":
             key = key[1:]
+            numkey = key.split(":")[0]
             keys = {"people": {}, "peoplesort": {}}
             if key in keys:
                 people = keys["people"]
@@ -218,35 +227,33 @@ class Collection(object):
                     self.__used.append(other)
                     self.__cache[other] = "\n".join(values)
                 return ret
-            elif key == "length":
-                length = self.__get_value("~#length")
+            elif numkey == "length":
+                length = self.__get_value("~#" + key)
                 return None if length is None else util.format_time(length)
-            elif key == "long-length":
-                length = self.__get_value("~#length")
-                if length is None:
-                    return None
-                else:
-                    return util.format_time_long(length)
-            elif key == "tracks":
-                tracks = self.__get_value("~#tracks")
+            elif numkey == "long-length":
+                length = self.__get_value("~#" + key[5:])
+                return (None if length is None
+                        else util.format_time_long(length))
+            elif numkey == "tracks":
+                tracks = self.__get_value("~#" + key)
                 return (None if tracks is None else
                         ngettext("%d track", "%d tracks", tracks) % tracks)
-            elif key == "discs":
-                discs = self.__get_value("~#discs")
+            elif numkey == "discs":
+                discs = self.__get_value("~#" + key)
                 if discs > 1:
                     return ngettext("%d disc", "%d discs", discs) % discs
                 else:
                     # TODO: check this is correct for discs == 1
                     return None
-            elif key == "rating":
-                rating = self.__get_value("~#rating")
+            elif numkey == "rating":
+                rating = self.__get_value("~#" + key)
                 if rating is None:
                     return None
                 return util.format_rating(rating)
             elif key == "cover":
                 return ((self.cover != type(self).cover) and "y") or None
-            elif key == "filesize":
-                size = self.__get_value("~#filesize")
+            elif numkey == "filesize":
+                size = self.__get_value("~#" + key)
                 return None if size is None else util.format_size(size)
             key = "~" + key
 
@@ -299,22 +306,16 @@ class Album(Collection):
         self.__dict__.pop("peoplesort", None)
         self.__dict__.pop("genre", None)
 
-    def scan_cover(self, force=False):
+    def scan_cover(self, force=False, scale_factor=1):
         if (self.scanned and not force) or not self.songs:
             return
         self.scanned = True
 
-        song = iter(self.songs).next()
-        cover = song.find_cover()
+        cover = cover_plugins.get_cover_many(self.songs)
 
         if cover is not None:
-            s = self.COVER_SIZE
-            try:
-                round = config.getboolean("albumart", "round")
-                self.cover = thumbnails.get_thumbnail(cover.name, (s, s))
-                self.cover = thumbnails.add_border(self.cover, 30, round)
-            except GLib.GError:
-                return
+            s = self.COVER_SIZE * scale_factor
+            self.cover = thumbnails.get_thumbnail_from_file(cover, (s, s))
 
     def __repr__(self):
         return "Album(%s)" % repr(self.key)
@@ -333,10 +334,13 @@ class Playlist(Collection, Iterable):
     unquote = staticmethod(unescape_filename)
 
     @classmethod
-    def new(cls, dir, base=_("New Playlist"), library={}):
-        if not (dir and os.path.realpath(dir)):
-            raise ValueError("Invalid playlist directory '%s'" % (dir,))
-        p = Playlist(dir, "", library=library)
+    def new(cls, dir_, base=_("New Playlist"), library=None):
+        assert is_fsnative(dir_)
+
+        if not (dir_ and os.path.realpath(dir_)):
+            raise ValueError("Invalid playlist directory %r" % (dir_,))
+
+        p = Playlist(dir_, "", library)
         i = 0
         try:
             p.rename(base)
@@ -350,7 +354,9 @@ class Playlist(Collection, Iterable):
         return p
 
     @classmethod
-    def fromsongs(cls, dir, songs, library={}):
+    def fromsongs(cls, dir_, songs, library=None):
+        assert is_fsnative(dir_)
+
         if len(songs) == 1:
             title = songs[0].comma("title")
         else:
@@ -360,13 +366,15 @@ class Playlist(Collection, Iterable):
                 len(songs) - 1) % (
                     {'title': songs[0].comma("title"),
                      'count': len(songs) - 1})
-        playlist = cls.new(dir, title, library=library)
+
+        playlist = cls.new(dir_, title, library)
         playlist.extend(songs)
         return playlist
 
     @classmethod
     def playlists_featuring(cls, song):
         """Returns the list of playlists in which this song appears"""
+
         playlists = []
         for instance in cls.__instances:
             if song in instance._list:
@@ -376,12 +384,16 @@ class Playlist(Collection, Iterable):
     # List-like methods, for compatibilty with original Playlist class.
     def extend(self, songs):
         self._list.extend(songs)
+        self.finalize()
 
     def append(self, song):
-        return self._list.append(song)
+        ret = self._list.append(song)
+        self.finalize()
+        return ret
 
     def clear(self):
         del self._list[:]
+        self.finalize()
 
     def __iter__(self):
         return iter(self._list)
@@ -397,6 +409,7 @@ class Playlist(Collection, Iterable):
 
     def __setitem__(self, key, value):
         self._list[key] = value
+        self.finalize()
 
     @property
     def songs(self):
@@ -411,19 +424,30 @@ class Playlist(Collection, Iterable):
 
         self.name = name
         self.dir = dir
+        self.library = library
         self._list = HashedList()
-        basename = self.quote(name)
         try:
-            for line in file(os.path.join(self.dir, basename), "r"):
-                line = util.fsnative(line.rstrip())
-                if line in library:
-                    self._list.append(library[line])
-                elif library and library.masked(line):
-                    self._list.append(line)
+            with open(self.filename, "rb") as h:
+                for line in h:
+                    assert library is not None
+                    try:
+                        line = bytes2fsnative(line.rstrip())
+                    except ValueError:
+                        # decoding failed
+                        continue
+                    if line in library:
+                        self._list.append(library[line])
+                    elif library and library.masked(line):
+                        self._list.append(line)
         except IOError:
             if self.name:
                 self.write()
 
+    @property
+    def filename(self):
+        basename = self.quote(self.name)
+        return os.path.join(self.dir, basename)
+
     def rename(self, newname):
         if isinstance(newname, unicode):
             newname = newname.encode('utf-8')
@@ -435,7 +459,7 @@ class Playlist(Collection, Iterable):
                 _("A playlist named %s already exists.") % newname)
         else:
             try:
-                os.unlink(os.path.join(self.dir, self.quote(self.name)))
+                os.unlink(self.filename)
             except EnvironmentError:
                 pass
             self.name = newname
@@ -449,14 +473,14 @@ class Playlist(Collection, Iterable):
                 changed = True
         return changed
 
-    def remove_songs(self, songs, library, leave_dupes=False):
+    def remove_songs(self, songs, leave_dupes=False):
         """Removes `songs` from this playlist if they are there,
          removing only the first reference if `leave_dupes` is True
         """
         changed = False
         for song in songs:
             # TODO: document the "library.masked" business
-            if library.masked(song):
+            if self.library is not None and self.library.masked(song):
                 while True:
                     try:
                         self._list[self.index(song)] = song("~filename")
@@ -472,6 +496,8 @@ class Playlist(Collection, Iterable):
                         break
                 else:
                     changed = True
+        if changed:
+            self.finalize()
         return changed
 
     def has_songs(self, songs):
@@ -488,20 +514,19 @@ class Playlist(Collection, Iterable):
     def delete(self):
         self.clear()
         try:
-            os.unlink(os.path.join(self.dir, self.quote(self.name)))
+            os.unlink(self.filename)
         except EnvironmentError:
             pass
         if self in self.__instances:
             self.__instances.remove(self)
 
     def write(self):
-        basename = self.quote(self.name)
-        with open(os.path.join(self.dir, basename), "w") as f:
+        with open(self.filename, "wb") as f:
             for song in self._list:
-                try:
-                    f.write(fsencode(song("~filename")) + "\n")
-                except TypeError:
-                    f.write(song + "\n")
+                if isinstance(song, basestring):
+                    f.write(fsnative2bytes(song) + "\n")
+                else:
+                    f.write(fsnative2bytes(song("~filename")) + "\n")
 
     def format(self):
         """Return a markup representation of information for this playlist"""
